/** The class LinProg contains static methods that can be used to
 *  solve a linear program. Most methods work on a 2 dimensional
 *  array called tableau.<br>
 *  This tableau is a concatenation of an:<br>
 *  <ol>
 *    <li> M_a x N_a matrix A containing the coefficients of an linear
 *  			 equation set (N_a variables, M_a equations).
 *		<li> M_a right hand value vector B.
 *    <li> N_a row vector C containing coefficients for target function.
 *    <li> Scalar R which is later the result of the optimisation.
 *    <li> /Schlupfvariablen/
 *  </ol>
 *  The dimensions of this tableau are: (M_a+1) x (N_a+M_a+1)
<PRE>
( C_0 , C_1 , ..., C_N  | 0,   ...  , 0 |  R  )
( ------------------------------------------- )
( A_11, A_12, ..., A_1N | 1, 0, ..  , 0 | B_1 )
(                       |               |     )
(        ... ...        |      ...      | ... )
(                       |               |     )
( A_M1, A_M2, ..., A_MN | 0,  ..   0, 1 | B_M )
</PRE>
 * <br>
 * Solving linear problems you first got to create the tableau shown
 * above. An easy way to do this is to call createTableau().<br>
 * The next step is to ensure to start with a valid solution. This is done
 * using the multiphase method. This method guarantees that alle b[i] are
 * greater than 0. For this method you need to pass an permutation vector.
 * You can generate one for your tableau using getPV(t), where t is your
 * tableau.<br>
 * At last you can run the SIMPLEX algorithm which finds the optimal solution
 * for Cx under restrictions defined by Ax = B.
 * For the result you will get the modified matrix, where -R is the optimum.
 */
public class LinProg {

	/** Creating tableau for linear program.
	 *  The tableau is build up as shown in the class description above.
	 *  @param c The target functions coefficients.
	 *  @param a The coefficient matrix of the linear equation set.
	 *  @param b The right hand side values.
	 *  @return The tableau for the linear program.
	 */
	public static double[][] createTableau(double[] c, double[][] a, double[] b) {
		int m = b.length + 1;
		int n = c.length + m;
		int colsOfA = a[0].length;
		double[][] t = new double[m][n];

		//puts target functions coefficients into tableau
		System.arraycopy(c,0,t[0],0,c.length);

		// puts the coefficient matrix, identity matrix and vector b
		// into the tableau
		for (int i=1; i<m; i++) {
			System.arraycopy(a[i-1],0,t[i],0,colsOfA);
			t[i][n-1] = b[i-1];
			t[i][colsOfA-1+i] = 1;
		}

		return t;
	}


	/** Returns an initialized permutaion vector for
	 *  the passed tableau.
	 *  @param tableau The tableau to create permutation vector from.
	 *  @return The initialized permutation vector pv[i] = i.
	 */
	public static int[] getPV(double[][] tableau) {
		int[] pv = new int[tableau[0].length -1];
		// filling indexfield
		for (int j=0; j<pv.length; j++) {
			pv[j] = j;
		}
		return pv;
	}

	/** Gauss elimination, using t[r][s] as pivot element.
	 *  The pivot element is scaled to be 1.0, all other
	 *  elements t[i][s] for in i != r, i e {0..A_m} will be eliminated.
   *  All operations are performed on the passed tableau.
	 *  @param t The tableau.
	 *  @param s The column of the pivot element.
	 *  @param r The row of the pivot element.
	 */
	protected static void eliminate (double[][] t, int s, int r) {
		int m = t.length;
		int n = t[0].length;

		double pivinv = 1/ t[r][s];
		double dum;

		for (int j=0; j<n; j++) t[r][j] *= pivinv;
		t[r][s] = 1;

		for (int i=0; i<m; i++) {
			if (i == r) continue;
			dum = t[i][s];
			for (int j=0; j<n; j++) {
				t[i][j] -= t[r][j]*dum;
			}
		}
	}


	/** Multiphase.
	 *  Ensures that all elements of the right hand side vector B in this
	 *  tableau are positiv. Operations are performed on the passed tableau.
	 *  @param t The tableau to check.
	 *  @param pv empty array of size n+1. Permutation indices are written
	 *         into this array.
	 *  @return Returns the logging information of the algorithmen.
	 *  @throw UnsolveableException if the linear program cannot be solved.
	 */
	public static String multiphase(double[][] t, int[] pv) {
		int m = t.length;
		int n = t[0].length;
		int indx;
		int s=-1;
		int r=-1;

		StringBuffer log = new StringBuffer("Starting Multiphase\n");

		indx = -1;
		for (int i=1; i<m; i++) {
			if (t[i][n-1] < 0) {
				// I.set(m-i);
				indx = i;
				break;
			}
		}

		int j;
		int maxs = m-2;
		// there are negative bvalues in b
		while (indx != -1) {
			j=0;
			while (t[indx][j] >= 0) {
				if (j==maxs) {
					throw new UnsolvableException("Empty /Loesungsraum/, because, t["+
					(indx) + "][i] .ge. 0 for all i (0...n-2) and " +
					"t["+ (indx) + "][n-1] is .lt. 0");
				}
				j++;
			}
			// pivot column
			s = j;

			double min = Double.POSITIVE_INFINITY;
			double tmp;
			for (int i=1; i<m; i++) {
				if (t[i][s] > 0) {
					tmp = t[i][n-1] / t[i][s];
					if (tmp < min) {
						min = tmp;
						r = i;
					}
				}
			}
			if (min == Double.POSITIVE_INFINITY) {
				// no positive value found
				r = s+1; // because 0 is c vector
			}

			eliminate(t,s,r);
			log.append("Choosing as Pivot - row = "+r+" col = "+s+"\n");
			log.append("After Pivot-Operation\n");
			log.append(getTableau(t));

			// do changes on the permutations vector
			int swap = pv[pv[s]];
			pv[pv[s]] = pv[n-m+r-1];
			pv[n-m+r-1] = swap;

			if (t[indx][n-1] < 0) {
				// try again for indx.
			} else {
				indx = -1;
				for (int i=1; i<m; i++) {
					if (t[i][n-1] < 0) {
						indx = i;
						break;
					}
				}
			}
		}
		log.append("Multiphase ended\n\n");
		return log.toString();

	}

	/** Checks if simplex has found the optimum.
	 *  If all C_i for i in [0..N_a +1] <= 0 it returns -1;
	 *  If C_i is > 0 then i is returned.
	 *  @param t The tableau
	 *  @return The first index in C where C_i is > 0 or -1 if the
	 *          optimum was found.
	 */
	private static int isResult(double[][] t) {
		int n = t[0].length-1;

		for (int i=0; i<n; i++) {
			if (t[0][i] > 0) return i;
		}
		return -1;
	}

	/** Simplex.
	 *  Runs Gauss Jordan algorithm (using special pivoting) on tableau to
	 *  determine the best solution for the target function c.
	 *  All operations are performed on the passed tableau. pv contains the
	 *  permutation of the results.
	 *  @param t The tableau.
	 *  @param pv empty array of size n+1. Permutation indices are written
	 *         into this array.
	 *  @return Returns the logging information of the algorithmen.
	 */
	public static String simplex(double[][] t, int[] pv) {
		int s = -1;
		int r = -1;
		int m = t.length;
		int n = t[0].length;
		StringBuffer log = new StringBuffer("Starting Simplex\n");

		while ((s=isResult(t)) != -1) {

			double min = Double.POSITIVE_INFINITY;
			double tmp;
			for (int i=1; i<m; i++) {
				if (t[i][s] > 0) {
					tmp = t[i][n-1] / t[i][s];
					if (tmp < min) {
						min = tmp;
						r = i;
					} else if (tmp == min) {
						int rOfNewMin = i;
						if (pv[n-m+rOfNewMin-1] < pv[n-m+r-1]) {
							r = rOfNewMin;
							min = tmp;
						}
					}
				}
			}
			if (min == Double.POSITIVE_INFINITY) {
				// no positive value found
				throw new UnsolvableException("unlimited, /Loesungsraum/");
			}

			eliminate(t,s,r);
			log.append("Choosing as Pivot - row = "+r+" col = "+s+"\n");
			log.append("After Pivot-Operation\n");
			log.append(getTableau(t));

			// do changes on the permutationsvector
			int swap = pv[pv[s]];
			pv[pv[s]] = pv[n-m+r-1];
			pv[n-m+r-1] = swap;

		}
		log.append("Sipmlex ended\n\n");
		return log.toString();
	}

	/** Prints the String that represents the tableau.
	 *  @param t The tableau to print
	 *  @return Returns the String that represents the tableau.
	 */
	public static String getTableau(double[][] t) {
		StringBuffer rv = new StringBuffer("Tableau = \n");
		for (int i=0;i<t.length;i++) {
			for (int j=0;j<t[0].length;j++) {
				double tmp =t[i][j]*1000;
				tmp = Math.round(tmp);
				tmp = tmp/1000;
				rv.append(tmp+"\t");
			}
			rv.append("\n");
		}
		return rv.toString();
	}


	/** Prints the result of the optimum and the
	 *  determined variables.
	 *  @param t The tableau
	 *  @param pv The permutationvector
	 *  @return Returns the optimum and the result of the
	 *   determined variables.
	 */
	public static String getResult(double[][] t, int[] pv) {
		StringBuffer sb = new StringBuffer();
		int m = t.length;
		int n = t[0].length;
		sb.append("\nfound optimum: " + (-t[0][n-1]) + " for:\n");

		int line=1;
		for (int i=(n-m);i<pv.length;i++) {
			sb.append("\tX["+pv[i]+"] = "+t[line][n-1]+"\n");
			line++;
		}
		sb.append("The other variables are 0.\n");
		return sb.toString();
	}
}